package lsedit;

// Layout method using Coffman-Graham Layer Assignment algorithm and the
// Sugiyama algorithm.

// John S. Y. Lee January 2000

import java.util.Enumeration;
import java.util.Vector;

public class SugiyamaLayout extends LandscapeLayouter  implements ToolBarEventHandler {

  public SugiyamaLayout(LandscapeViewerCore ls) 
  {
	super(ls);
  }

/****************************************************************************/

  public String getMenuLabel() 
  {
	return "Coffman-Graham-Sugiyama method applied";
  } // getMenuLabel

/***************************************************************************/

  // The boxIndex method returns the vector index of a box named ID.

  private static int boxIndex(Vector selectedBoxes, String ID) {
	
	int index;
	for (index = 0; index < selectedBoxes.size(); index++) {
	  if (ID.equals(((EntityInstance)selectedBoxes.elementAt(index)).getId()))
		break;
	} // for
	return index;

  } // boxIndex

/***************************************************************************/

  // The doLayout method executes the Coffman-Graham Layer Assignment
  // algorithm and the Sugiyama algorithm on the boxes selected.
  // Assumption: All boxes selected are in the same container.

  // This is called directly to layout incoming TA

	public static void doLayout1(Vector selectedBoxes, boolean update) 
	{
		Enumeration		en;
		EntityInstance	e;
		boolean			debug = false;
		int				vertexID = 0;
		int				groupSize = selectedBoxes.size();

		// create graph to store info on selected boxes and their relationships
		Graph graph = new Graph(groupSize);

		for (en = selectedBoxes.elements(); en.hasMoreElements(); ) {
			e = (EntityInstance) en.nextElement();
			graph.addVertex(e.widthRelLocal());
		}

		// get (directed) relationships between the boxes

		for (en = selectedBoxes.elements(); en.hasMoreElements(); ++vertexID) {
			e = (EntityInstance) en.nextElement();

			for (Enumeration edge = e.srcRelListEnum(); edge.hasMoreElements(); ) {
				RelationInstance relation = (RelationInstance) edge.nextElement();
				RelationClass	 rc       = (RelationClass)    relation.getParentClass();
				if (rc != null && !rc.isClassVisible()) {
					continue;
				}
				EntityInstance child = relation.getDst();
			  
				// store relationship only if children is in the same container

				int index = boxIndex(selectedBoxes, child.getId());
				if (index < groupSize) {
					graph.addEdge(vertexID, index);
				}
			} // for
		} // for

		if (debug) {
			int index = 0;
		  
			System.out.println("Graph read:");
			for (en = selectedBoxes.elements(); en.hasMoreElements(); index++) {
				System.out.println(index + " " + en.nextElement());
			}
			graph.print();
		}

		Vector layers = graph.doCoffmanGrahamSugiyama();

		if (debug) {
			System.out.println("RESULTS");
			int level = 0;
			for (en = layers.elements(); en.hasMoreElements(); ) {
				System.out.println("level " + level);
				for (Enumeration f = ((Vector)en.nextElement()).elements(); f.hasMoreElements(); ) {
					System.out.print(f.nextElement() + " ");
				}
				System.out.println();
			}
		} // if

		// Assign coordinates to each box

		int		numLayers   = layers.size();
		double	layerHeight = (double) 1.0 / (double)numLayers;
		double	xrel, yrel, widthrel, heightrel, boxWidth;
		Vector	row;

		double curX = 0.01;
		double curY = 1.0;

		for (en = layers.elements(); en.hasMoreElements(); ) {

			row       = (Vector) en.nextElement();
			boxWidth  = (double) 1.0 / (double)(row.size());

			for (Enumeration f = row.elements(); f.hasMoreElements(); ) {
				int curVertex = ((Integer)f.nextElement()).intValue();

				// Process non-dummy boxes only
				if (curVertex < groupSize) {
					e         = (EntityInstance) selectedBoxes.elementAt(curVertex);
					widthrel  = e.widthRelLocal();
					heightrel = e.heightRelLocal();
					xrel = curX + (boxWidth    - widthrel) / 2;
					yrel = curY - (layerHeight - heightrel)/ 2;

					if (update) {
						e.updateRelLocal(xrel, yrel, widthrel, heightrel);
					} else {
					  e.setxRelLocal(xrel);
					  e.setyRelLocal(yrel);
					}
					curX += boxWidth;
				} // if
			} // for

			curY -= layerHeight;
			curX  = 0.01;

		} // for

	} // doLayout


  // The doLayout method executes the Coffman-Graham Layer Assignment
  // algorithm and the Sugiyama algorithm on the boxes selected.
  // Assumption: All boxes selected are in the same container.

	public String doLayout(Diagram dg) {

		boolean debug = true;

		// get user's selection of boxes to be laid out

		Vector selectedBoxes = dg.getGroup();
		if (selectedBoxes == null) {
			  beep();
			  return "No group selected";
		}

		dg.beginUndoRedo("Sugiyama layout");
		doLayout1(selectedBoxes, true /* Updating */);
		dg.endUndoRedo();
		return "Graph redrawn with Coffman-Graham-Sugiyama method";

	} // doLayout

	public void processKeyEvent(int key, int modifiers, Object object) 
	{
		Diagram	dg;
		String	rmsg;

		dg = m_ls.getDiagram();
		if (dg != null) {
			rmsg = doLayout(dg);
			m_ls.doFeedback(rmsg);
			dg.rescaleDiagram();
			m_ls.repaintDg();
	}	}

} // SugiyamaLayout






